;*********************************************************************
; COMMODORE VIC 20 BOOT USING BASIC 2.0
; written by Robert Hurst <robert@hurst-ri.us>
; updated version: 31-Oct-2010
;
		.fileopt author,	"your name"
        .fileopt comment,	"your program name"
        .fileopt compiler,	"VIC 20 ASSEMBLER"

		.include "VIC-SSS-MMX.h"

;*********************************************************************
; VIC Software Sprite Stack 2010 (VIC-SSS-MMX)
;
; The above BASIC loader will be overwritten by SSS upon its
; initialization (SSSINIT).  The linker will fill this reserved space
; with values used for the dual video frame buffers, play field, and
; the sprite image buffers and registers: 4096 - 6207 ($1000 - $1BFF)
;
; $1000 - $11FF		VICFRAME1 - first video buffer
; $1200 - $13FF		VICFRAME2 - second video buffer
; $1400 - $15FF		PLAYFIELD - write-pending screen buffer
; $1600 - $17FF		PLAYCOLOR - write-pending color / dirty buffer
; $1800 - $1BFF		Sprite image buffers & registers
;
			.segment "SSSBUF"

SSSBUF:		.res 98 * 8		; this can be resized as required --
							; if all 64-chars are used by sprites, that
							; exhausts all 128 custom characters for
							; double-buffering (x2)
;
; SPRITE REGISTERS (17)
;
SPRITEBACK:	.res SPRITEMAX	; 1st char this sprite is in collision with
SPRITEBUFH:	.res SPRITEMAX	; pointer within sprite image buffer
SPRITEBUFL:	.res SPRITEMAX	; pointer within sprite image buffer
SPRITEC1H:	.res SPRITEMAX	; pointer within sprite display character pool
SPRITEC1L:	.res SPRITEMAX	; pointer within sprite display character pool
SPRITEC2H:	.res SPRITEMAX	; pointer within sprite display character pool
SPRITEC2L:	.res SPRITEMAX	; pointer within sprite display character pool
SPRITECOL:	.res SPRITEMAX	; 4-bit VIC color code
SPRITECX:	.res SPRITEMAX	; sprite collision X-coord
SPRITECY:	.res SPRITEMAX	; sprite collision Y-coord
SPRITEDEF:	.res SPRITEMAX	; function/matrix definition (see explanation below)
SPRITEDEF2:	.res SPRITEMAX	; extended sprite definition
SPRITEH:	.res SPRITEMAX	; number of raster lines (1-16)
SPRITEIMGH:	.res SPRITEMAX	; pointer to source graphic for rendering at 0,0
SPRITEIMGL:	.res SPRITEMAX	; pointer to source graphic for rendering at 0,0
SPRITEX:	.res SPRITEMAX	; horizontal pixel coordinate, visible >0 - <SSSCLIPX
SPRITEY:	.res SPRITEMAX	; vertical pixel coordinate, visible >0 - <SSSCLIPY
SPRITEZ:	.res SPRITEMAX	; bit 0: last rendered (0 = SPRITEC1; 1 = SPRITEC2)
							; bit 1: fast copy (0 = merge; 1 = copy)
							; bit 3: sprite collision
							; bit 4: sprite image is clipped by a static cell
							; bit 5: background is all SSSNULLs
							; bit 6: copy/merge into alternate sprite character pool
							; bit 7: copy/shift sprite image into its buffer
SSSZORDR:	.res SPRITEMAX

;SPRITEDEF		; matrix definition:
						; bit 0: unused
						; bit 1: unused
						; bit 2: unused
						; bit 3: unused
						; bit 4: repeat		0=independent; 1=re-use previous
						; bit 5: ghost		0=merge image; 1=invert image
						; bit 6: collision	0=ignore; 1=detect
						; bit 7: enabled	0=invisible; 1=visible

;SPRITEDEF2		; matrix definition:
						; bit 0: unused
						; bit 1: width		0 = 8px; 1 = 16px
						; bit 2: float Y	0=fixed cell; 1=vertical float
						; bit 3: float X	0=fixed cell; 1=horizontal float
						; bit 4: height		00 = 8, 01 =16, 10 = 24, 11=Unsupported
						; bit 5: height		00 = 8, 01 =16, 10 = 24, 11=Unsupported
						; bit 6: unused
						; bit 7: unused


SPRITES:	.byte 0		; number of active sprite registers (0 - SPRITEMAX)
						; SSS runtime variables:
sss:		.res 24*2	; screen row index, computed from PLAYCOLS in SSSINIT
sssIDX:		.byte 0		; current sprite loop index# (0-SPRITEMAX)
sssNUM:		.byte 0		; current sprite being drawn# (0-SPRITEMAX)
sssX:		.byte 0		; current sprite width: 0=8w, 1=16w, 2=24w
sssY:		.byte 0		; current sprite height: 0=8h, 1=16h, 2=24h
sssBYTES:	.byte 0		; number of bytes this sprite occupies
sssNEXT:	.byte 0		; offset to adjacent character
sssCHAR:	.byte 0		; next custom character to use on PENDING frame
sssDX:		.byte 0		; delta X counter
sssDY:		.byte 0		; delta Y counter
sssLINE:	.byte 0		; current sprite make line: 0, 8, 16
sssLINENUM:	.byte 0		; current sprite line countdown
sssROR1:	.byte 0		; bit shift register column #1
sssROR2:	.byte 0		; bit shift register column #2
.ifdef SPRITEWIDE
sssROR3:	.byte 0		; bit shift register column #3
.endif
sssXFER:	.byte 0		; transfer to custom character counter
sssMC:		.byte 0		; is scurrent sprite MC?
sssSPRITEDDEF2: .byte 0 ; store extended sprite definition
sssHEIGHTS: .byte 8,16,24;
sssMCMASKS: .byte 0, %11000000, %00110000, %00001100, %00000011

;
; other initialized data can be appended here:
;
			.segment "RODATA"
			;              H x W [0 = configuration not supported]
sssALLOC:	;       8x8, 16x8, 24x8, [32x8], 8x16, 16x16, 24x16, [32x 16] 
			.byte	8,16,24,0,16,32,48,0    ; fixed:	    1,2,3,n/a,2,4,6,n/a
			.byte	16,24,32,0,32,48,64,0   ; float Y:	    2,3,4,n/a,4,6,8,n/a
			.byte	16,32,48,0,24,48,72,0   ; float X:	    2,4,6,n/a,3,6,9,n/a
			.byte	32,48,64,0,48,72,96,0   ; float both:	    4,6,8,n/a,9,6,12,n/a
sssROWS:	.byte	1,2,3,0,1,2,3,0	; fixed
			.byte	2,3,4,0,2,3,4,0	; float Y
			.byte	1,2,3,0,1,2,3,0	; float X
			.byte	2,3,4,0,2,3,4,0	; float XY
sssCOLS:	.byte	1,1,1,0,2,2,2,0	; fixed
			.byte	1,1,1,0,2,2,2,0	;  float Y
			.byte	2,2,2,0,3,3,3,0	;  float X
			.byte	2,2,2,0,3,3,3,0	;  float XY



	.segment "RODATA2"			

; LOOKUP TABLES ----------------------------

ASL5DATA:
.repeat 256 , lc
	.byte   <((<lc-$18) * 32)
.endrepeat
ASL5 = ASL5DATA
.global ASL5			

	.segment "SSSBUF"		
	
LSR3LESS16DATA:
.repeat 256 , lc
	.byte   <((<lc-$10) >> 3)
.endrepeat
LSR3LESS16 = LSR3LESS16DATA
.global LSR3LESS16			

	.segment "DATA"	

SSSIMAGEBUFX: .res 256,0	
SSSIMAGEBUFY: .res 256,0	
.global SSSIMAGEBUFX
.global SSSIMAGEBUFY